@@ -39,6 +39,9 @@ extern "C" {
 
 #ifndef SIMPLE_CODE
 
+#include <netinet/in.h>
+
+#include <fstream>
 #include <iomanip>
 #include <sstream>
 
@@ -47,9 +50,13 @@ extern "C" {
 
 #include <Lum/Base/DateTime.h>
 #include <Lum/Base/L10N.h>
+#include <Lum/Base/Path.h>
 #include <Lum/Base/String.h>
 
 #include <Lum/Dlg/About.h>
+#include <Lum/Dlg/File.h>
+#include <Lum/Dlg/Msg.h>
+#include <Lum/Dlg/Value.h>
 
 #include <Lum/Model/Action.h>
 #include <Lum/Model/Boolean.h>
@@ -59,7 +66,7 @@ extern "C" {
 
 #include <Lum/OS/Main.h>
 
-#include <Lum/AppInfo.h>
+#include <Lum/Button.h>
 #include <Lum/Dialog.h>
 #include <Lum/Label.h>
 #include <Lum/LED.h>
@@ -85,6 +92,7 @@ GMainLoop    *loop = NULL;
 static long obsoleteDataAge = 30; // maximum ge of data in seconds until value is invalid
 static long reconnectTime   = 10; // Period of time after which we check for disconnection and
                                   // trigger reconnection
+static long refreshTime     = 10; // Period of time after which we update the stream views
 
 class GPSData
 {
@@ -465,8 +473,10 @@ public:
 };
 
 class MyWindow;
-static MyWindow* window=NULL;
-static Lum::AppInfo info;
+static MyWindow*         window=NULL;
+static Lum::Def::AppInfo info;
+static unsigned long     trackingFileId='G'*256*256*256+'J'*256*256+'T'*256+'\0';
+static unsigned long     trackingFileVersion=1;
 #endif
 
 #if defined(HAVE_LIB_GPS)
@@ -497,6 +507,67 @@ std::wstring DoubleToString(double value)
   }
 }
 
+class Track
+{
+private:
+  std::wstring          name;
+  Lum::Base::SystemTime time;
+
+public:
+  Track(const std::wstring& name, const Lum::Base::SystemTime& time)
+  : name(name),
+    time(time)
+  {
+    // no code
+  }
+
+  std::wstring GetName() const
+  {
+    return name;
+  }
+
+  std::wstring GetTimeString() const
+  {
+    return time.GetLocalLocaleDate()+L" "+time.GetLocalLocaleTime();
+  }
+};
+
+
+typedef Lum::Model::StdTable<Track> TracksModel;
+typedef Lum::Base::Reference<TracksModel> TracksModelRef;
+
+class TracksDataProvider : public TracksModel::DataProvider
+{
+public:
+  std::wstring GetString(const TracksModel::Iterator& iter, size_t column) const
+  {
+    switch (column) {
+    case 1:
+      return iter->GetName();
+    case 2:
+      return iter->GetTimeString();
+    default:
+      assert(false);
+    }
+  }
+};
+
+class TrackComparator : public TracksModel::Comparator
+{
+public:
+  bool operator()(const Track& a, const Track& b, size_t column, bool down) const
+  {
+    assert(column==1);
+
+    if (down) {
+      return a.GetName()<b.GetName();
+    }
+    else {
+      return a.GetName()>b.GetName();
+    }
+  }
+};
+
 class MyWindow : public Lum::Dialog
 {
 private:
@@ -514,13 +585,23 @@ private:
   SatellitesModelRef       satellites;
   Lum::Model::DoubleDataStreamRef speedStream;
   Lum::Model::DoubleDataStreamRef altitudeStream;
+  TracksModelRef           tracks;
+  Lum::Model::SingleLineSelectionRef trackSelection;
 
+  Lum::Model::ActionRef    startTrackingAction;
+  Lum::Model::ActionRef    stopTrackingAction;
+  Lum::Model::ActionRef    deleteTrackAction;
+  Lum::Model::ActionRef    exportTrackAction;
   Lum::Model::ActionRef    aboutAction;
 
   Lum::Model::ActionRef    dataChangedAction;
   Lum::Model::ActionRef    refreshTimer;
   Lum::Model::ActionRef    reconnectCheckTimer;
 
+  Lum::Model::BooleanRef   tracking;
+
+  std::ofstream            trackFile;
+
 #if defined(HAVE_LIB_LOCATION)
   LocationGPSDControl      *control;
 #endif
@@ -550,9 +631,7 @@ private:
 #if defined(HAVE_LIB_LOCATION)
     control=location_gpsd_control_get_default();
 
-    if(control->can_control) {
-      location_gpsd_control_start(control);
-    }
+    location_gpsd_control_start(control);
 #endif
 #if defined(HAVE_LIB_GPS)
     // This fixed a bug in libgps, using locale aware functions to parse non-locale-aware float values
@@ -582,9 +661,7 @@ private:
 #endif
 
 #if defined(HAVE_LIB_LOCATION)
-    if(control->can_control) {
-      location_gpsd_control_stop(control);
-    }
+    location_gpsd_control_stop(control);
 
     /*Device control cleanup */
     g_object_unref(control);
@@ -636,10 +713,17 @@ public:
     satellites(new SatellitesModel(new SatellitesDataProvider())),
     speedStream(new Lum::Model::DoubleDataStream()),
     altitudeStream(new Lum::Model::DoubleDataStream()),
+    tracks(new TracksModel(new TracksDataProvider(),new TrackComparator())),
+    trackSelection(new Lum::Model::SingleLineSelection()),
+    startTrackingAction(new Lum::Model::Action()),
+    stopTrackingAction(new Lum::Model::Action()),
+    deleteTrackAction(new Lum::Model::Action()),
+    exportTrackAction(new Lum::Model::Action()),
     aboutAction(new Lum::Model::Action()),
     dataChangedAction(new Lum::Model::Action()),
     refreshTimer(new Lum::Model::Action()),
-    reconnectCheckTimer(new Lum::Model::Action())
+    reconnectCheckTimer(new Lum::Model::Action()),
+    tracking(new Lum::Model::Boolean(false))
 #if defined(HAVE_LIB_LOCATION)
     ,control(NULL)
 #endif
@@ -648,12 +732,28 @@ public:
 #endif
   {
     Observe(GetOpenedAction());
+    Observe(startTrackingAction);
+    Observe(stopTrackingAction);
+    Observe(deleteTrackAction);
+    Observe(exportTrackAction);
     Observe(aboutAction);
+
     Observe(dataChangedAction);
     Observe(refreshTimer);
     Observe(reconnectCheckTimer);
+    Observe(trackSelection);
 
     ::window=this;
+
+    startTrackingAction->Enable();
+    stopTrackingAction->Disable();
+  }
+
+  ~MyWindow()
+  {
+    if (tracking->Get()) {
+      StopTracking();
+    }
   }
 
   void PreInit()
@@ -671,9 +771,13 @@ public:
 
     label=Lum::Label::Create(true,true);
 
-    label->AddLabel(L"Connected:",new Lum::LED(connected));
-    label->AddLabel(L"Online:",new Lum::LED(online));
-    label->AddLabel(L"Fix:",new Lum::LED(fix));
+    label->AddLabel(L"Conn./Online/Fix:",
+                    Lum::HPanel::Create()
+                    ->Add(new Lum::LED(connected))
+                    ->AddSpace()
+                    ->Add(new Lum::LED(online))
+                    ->AddSpace()
+                    ->Add(new Lum::LED(fix)));
     label->AddLabel(L"Time:",CreateTextValue(time,24)); // TODO: Measure by example
     label->AddLabel(L"Latitude:",CreateTextValue(latitude,10));
     label->AddLabel(L"Longitude:",CreateTextValue(longitude,10));
@@ -705,16 +809,52 @@ public:
 
     tab->Add(L"Compass",compass);
 
-    tab->Add(L"History",
+    tab->Add(L"Diagrams",
              Lum::VPanel::Create(true,true)
-             ->Add(new Lum::Text(L"Speed (m/s)"))
+             ->Add(Lum::Text::Create(L"Speed (m/s)"))
              ->AddSpace()
              ->Add(Lum::View::Create(TimeLine::Create(speedStream,true,true),true,true))
              ->AddSpace()
-             ->Add(new Lum::Text(L"Altitude (m)"))
+             ->Add(Lum::Text::Create(L"Altitude (m)"))
              ->AddSpace()
              ->Add(Lum::View::Create(TimeLine::Create(altitudeStream,true,true),true,true)));
 
+    headerModel=new Lum::Model::HeaderImpl();
+    headerModel->AddColumn(L"Track",Lum::Base::Size::stdCharWidth,10,true);
+    headerModel->AddColumn(L"Time",Lum::Base::Size::stdCharWidth,19); // TODO
+
+    table=new Lum::Table();
+    table->SetFlex(true,true);
+    table->SetShowHeader(true);
+    table->SetHeaderModel(headerModel);
+    table->SetModel(tracks);
+    table->SetSelection(trackSelection);
+    table->GetTableView()->SetAutoFitColumns(true);
+
+    tab->Add(L"Tracking",
+             Lum::VPanel::Create(true,true)
+             ->Add(Lum::HPanel::Create(true,false)
+                   ->Add(Lum::Button::Create(L"Record",
+                                             Lum::OS::Theme::imageMediaRecord,
+                                             startTrackingAction,
+                                             Lum::Button::typeIcon))
+                   ->Add(Lum::Button::Create(L"Stop",
+                                             Lum::OS::Theme::imageMediaStop,
+                                             stopTrackingAction,
+                                             Lum::Button::typeIcon))
+                   ->AddSpace(true)
+                    ->Add(new Lum::LED(tracking))
+                    ->AddSpace(true)
+                   ->Add(Lum::Button::Create(L"_Delete",
+                                             Lum::OS::Theme::imageRemove,
+                                             deleteTrackAction,
+                                             Lum::Button::typeIcon))
+                   ->Add(Lum::Button::Create(L"_Export",
+                                             Lum::OS::Theme::imageSave,
+                                             exportTrackAction,
+                                             Lum::Button::typeIcon)))
+             ->AddSpace()
+             ->Add(table));
     SetMain(tab);
 
     Lum::MenuDesc *menu=new Lum::MenuDesc();
@@ -739,6 +879,380 @@ public:
     data=this->data;
   }
 
+  void StartTracking()
+  {
+    Lum::Model::StringRef trackName=new Lum::Model::String(L"");
+    Lum::Base::Path       fileName;
+
+    if (!Lum::Dlg::TextInput::GetText(GetWindow(),
+                                      L"Enter track name!",
+                                      L"Please enter a name for the to be recorded track.",
+                                      trackName) || trackName->Empty()) {
+      return;
+    }
+
+    fileName.SetNativeDir(Lum::Base::Path::GetHome());
+    fileName.AppendDir(L".gpsjinni");
+
+    Lum::Base::Path::CreateDir(fileName.GetPath());
+
+    fileName.SetBaseName(trackName->Get()+L".gjt");
+
+    if (Lum::Base::Path::Exists(fileName.GetPath())) {
+      Lum::Dlg::Msg::ShowOk(GetWindow(),
+                            L"Error while opening file!",
+                            std::wstring(L"An error occured while opening file:\n")+
+                            L"'"+fileName.GetPath()+L"'\n"+
+                            L"Error:\n"+
+                            L"'File already exists'");
+      return;
+    }
+
+    trackFile.open(Lum::Base::WStringToString(fileName.GetPath()).c_str(),
+                   std::ios::out|std::ios::trunc|std::ios::binary);
+
+    if (!trackFile) {
+      Lum::Base::Status status;
+
+      status.SetToCurrentErrno();
+      Lum::Dlg::Msg::ShowOk(GetWindow(),
+                            L"Error while opening file!",
+                            std::wstring(L"An error occured while opening file:\n")+
+                            L"'"+fileName.GetPath()+L"'\n"+
+                            L"Error:\n"+
+                            L"'"+status.GetDescription()+L"'");
+
+      return;
+    }
+
+    unsigned long tfid=htonl(trackingFileId);
+    unsigned long tfv=htonl(trackingFileVersion);
+    unsigned long tft=htonl(Lum::Base::SystemTime().GetTime());
+
+    trackFile.write((const char*)&tfid,sizeof(tfid));
+    trackFile.write((const char*)&tfv,sizeof(tfv));
+    trackFile.write((const char*)&tft,sizeof(tft));
+
+    startTrackingAction->Disable();
+    stopTrackingAction->Enable();
+    tracking->Set(true);
+  }
+
+  void AddTrackingRecord(const GPSData& data)
+  {
+    if (!tracking->Get()) {
+      return;
+    }
+
+    double                lon=0.0;
+    double                lat=0.0;
+    double                alt=0.0;
+    Lum::Base::SystemTime tim;
+
+    if (data.HasLongitude()) {
+      lon=data.GetLongitude();
+    }
+
+    if (data.HasLatitude()) {
+      lat=data.GetLatitude();
+    }
+
+    if (data.HasAltitude()) {
+      alt=data.GetAltitude();
+    }
+
+    if (data.HasTime()) {
+      tim=data.GetTime();
+    }
+
+    unsigned long flon=htonl(round((lon+90.0)*10000000.0));
+    unsigned long flat=htonl(round((lat+180.0)*10000000.0));
+    unsigned long falt=htonl(round(alt*100));
+    unsigned long ftim=htonl(tim.GetTime());
+
+    //std::cout << "TrackRecord: " << std::hex << flon << " " << flat << " " << falt << " " << ftim << std::endl;
+
+    trackFile.write((const char*)&flon,sizeof(flon));
+    trackFile.write((const char*)&flat,sizeof(flat));
+    trackFile.write((const char*)&falt,sizeof(falt));
+    trackFile.write((const char*)&ftim,sizeof(ftim));
+  }
+
+  void StopTracking()
+  {
+    trackFile.close();
+
+    startTrackingAction->Enable();
+    stopTrackingAction->Disable();
+    tracking->Set(false);
+
+    RefreshTracks();
+  }
+
+  void RefreshTracks()
+  {
+    Lum::Base::Path              dirName,fileName;
+    Lum::Base::DirScannerRef     scanner;
+    Lum::Base::DirScanner::Entry entry;
+    Lum::Base::Status            status;
+
+    tracks->Clear();
+    tracks->Off();
+
+    dirName.SetNativeDir(Lum::Base::Path::GetHome());
+    dirName.AppendDir(L".gpsjinni");
+
+    scanner=Lum::Base::DirScanner::Create(dirName);
+
+    while (scanner->GetNext(entry,status)) {
+      if (entry.isDir) {
+        continue;
+      }
+
+      std::cout << "Found possible track '" << Lum::Base::WStringToString(entry.name) << "'" << std::endl;
+
+      if (entry.name.length()<4 ||
+          entry.name.substr(entry.name.length()-4)!=L".gjt") {
+        std::cerr << "wrong suffix!" << std::endl;
+        continue;
+      }
+
+      std::ifstream file;
+
+      fileName=dirName;
+      fileName.SetBaseName(entry.name);
+
+      file.open(Lum::Base::WStringToString(fileName.GetPath()).c_str(),
+                std::ios::in|std::ios::binary);
+
+      if (!file) {
+        std::cerr << "Cannot open file!" << std::endl;
+        continue;
+      }
+
+      unsigned long tfid,tfv,tft;
+
+      file.read((char*)&tfid,sizeof(tfid));
+      file.read((char*)&tfv,sizeof(tfv));
+      file.read((char*)&tft,sizeof(tft));
+
+      if (!file) {
+        std::cerr << "Cannot read file header!" << std::endl;
+        continue;
+      }
+
+      tfid=ntohl(tfid);
+      tfv=ntohl(tfv);
+      tft=ntohl(tft);
+
+      if (tfid!=trackingFileId) {
+        std::cerr << "Wrong magic!" << std::endl;
+        continue;
+      }
+
+      if (tfv!=trackingFileVersion) {
+        std::cerr << "Wrong file version!" << std::endl;
+        continue;
+      }
+
+      tracks->Append(Track(entry.name.substr(0,entry.name.length()-4),
+                           Lum::Base::SystemTime(tft)));
+    }
+
+    tracks->Sort(1);
+
+    tracks->On();
+  }
+
+  void ExportTrack(const std::wstring& name)
+  {
+    Lum::Base::Path                path;
+    std::wstring                   file;
+    Lum::Dlg::File::Options        *options=new Lum::Dlg::File::Options();
+    Lum::Model::Dir::PatternFilter *filter;
+    std::ifstream                  in;
+    std::ofstream                  out;
+
+    path.SetNativeDir(Lum::Base::Path::GetHome());
+    path.AppendDir(L".gpsjinni");
+    path.SetBaseName(name+L".gjt");
+
+    in.open(Lum::Base::WStringToString(path.GetPath()).c_str(),
+            std::ios::in|std::ios::binary);
+
+    if (!in) {
+      Lum::Base::Status status;
+
+      status.SetToCurrentErrno();
+      Lum::Dlg::Msg::ShowOk(GetWindow(),
+                            L"Error while opening file!",
+                            std::wstring(L"An error occured while opening file:\n")+
+                            L"'"+file+L"'\n"+
+                            L"Error:\n"+
+                            L"'"+status.GetDescription()+L"'");
+
+      return;
+    }
+
+    unsigned long tfid,tfv,tft;
+
+    in.read((char*)&tfid,sizeof(tfid));
+    in.read((char*)&tfv,sizeof(tfv));
+    in.read((char*)&tft,sizeof(tft));
+
+    if (!in) {
+      std::cerr << "Cannot read file header!" << std::endl;
+      return;
+    }
+
+    tfid=ntohl(tfid);
+    tfv=ntohl(tfv);
+    tft=ntohl(tft);
+
+    if (tfid!=trackingFileId) {
+      Lum::Dlg::Msg::ShowOk(GetWindow(),
+                            L"Error while opening file!",
+                            std::wstring(L"An error occured while opening file:\n")+
+                            L"'"+file+L"'\n"+
+                            L"Error:\n"+
+                            L"'File is not a stored track'");
+      return;
+    }
+
+    if (tfv!=trackingFileVersion) {
+      Lum::Dlg::Msg::ShowOk(GetWindow(),
+                            L"Error while opening file!",
+                            std::wstring(L"An error occured while opening file:\n")+
+                            L"'"+file+L"'\n"+
+                            L"Error:\n"+
+                            L"'File format version not supported'");
+      return;
+    }
+
+    path.SetNativeDir(Lum::Base::Path::GetHome());
+    path.SetBaseName(name+L".kml");
+
+    file=path.GetPath();
+
+    options->SetMode(Lum::Dlg::File::Options::modeFile);
+    options->SetExistingOnly(false);
+
+    filter=new Lum::Model::Dir::PatternFilter(L"KML Files");
+    filter->AddPattern(L"*.kml");
+    options->AddFilter(filter);
+
+    if (!Lum::Dlg::File::SaveFile(GetWindow(),
+                                  L"Exporting track...",
+                                  file,
+                                  options)) {
+      return;
+    }
+
+    std::cout << "Saving to '" << Lum::Base::WStringToString(file) << "'" << std::endl;
+
+    out.open(Lum::Base::WStringToString(file).c_str(),
+             std::ios::out|std::ios::trunc);
+
+    if (!out) {
+      Lum::Base::Status status;
+
+      status.SetToCurrentErrno();
+      Lum::Dlg::Msg::ShowOk(GetWindow(),
+                            L"Error while opening file!",
+                            std::wstring(L"An error occured while opening file:\n")+
+                            L"'"+file+L"'\n"+
+                            L"Error:\n"+
+                            L"'"+status.GetDescription()+L"'");
+
+      return;
+    }
+
+    out << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << std::endl;
+    out << "<kml xmlns=\"http://www.opengis.net/kml/2.2\">" << std::endl;
+    out << "  <Document>" << std::endl;
+    out << "    <name>" << Lum::Base::WStringToUTF8(name) << ".kml" << "</name>" << std::endl;
+    out << "    <open>1</open>" << std::endl;
+    out << "    <Style id=\"lineStyle\">" << std::endl;
+    out << "      <LineStyle>" << std::endl;
+    out << "        <color>7f0000ff</color>" << std::endl;
+    out << "        <width>4</width>" << std::endl;
+    out << "      </LineStyle>" << std::endl;
+    out << "    </Style>" << std::endl;
+
+    out << "    <Placemark>" << std::endl;
+    out << "      <name>" << Lum::Base::WStringToUTF8(name) << "</name>" << std::endl;
+    out << "      <styleUrl>#lineStyle</styleUrl>" << std::endl;
+    out << "      <LineString>" << std::endl;
+    out << "        <extrude>0</extrude>" << std::endl;
+//    out << "        <tessellate>1</tessellate>" << std::endl;
+    out << "        <altitudeMode>absolute</altitudeMode>" << std::endl;
+    out << "        <coordinates>" << std::endl;
+
+    while (in) {
+      unsigned long flon;
+      unsigned long flat;
+      unsigned long falt;
+      unsigned long ftim;
+
+      in.read((char*)&flon,sizeof(flon));
+      in.read((char*)&flat,sizeof(flat));
+      in.read((char*)&falt,sizeof(falt));
+      in.read((char*)&ftim,sizeof(ftim));
+
+      if (in) {
+        double                lon;
+        double                lat;
+        double                alt;
+        Lum::Base::SystemTime time;
+
+        flon=ntohl(flon);
+        flat=ntohl(flat);
+        falt=ntohl(falt);
+        ftim=ntohl(ftim);
+
+        lon=flon/10000000.0-90.0;
+        lat=flat/10000000.0-180.0;
+        alt=falt/100;
+        time.SetTime(ftim);
+
+        out << "          " << lon << "," << lat << "," << alt << std::endl;
+      }
+    }
+
+    out << "        </coordinates>" << std::endl;
+    out << "      </LineString>" << std::endl;
+    out << "    </Placemark>" << std::endl;
+    out << "  </Document>" << std::endl;
+    out << "</kml>" << std::endl;
+
+    out.close();
+    in.close();
+  }
+
+  void DeleteTrack(const std::wstring& name)
+  {
+    Lum::Base::Path   path;
+    Lum::Base::Status status;
+
+    path.SetNativeDir(Lum::Base::Path::GetHome());
+    path.AppendDir(L".gpsjinni");
+    path.SetBaseName(name+L".gjt");
+
+    status=Lum::Base::Path::RemoveFile(path.GetPath());
+
+    if (status) {
+      RefreshTracks();
+    }
+    else {
+      Lum::Dlg::Msg::ShowOk(GetWindow(),
+                            L"Error while deleting file!",
+                            std::wstring(L"An error occured while deletion of file:\n")+
+                            L"'"+path.GetPath()+L"'\n"+
+                            L"Error:\n"+
+                            L"'"+status.GetDescription()+L"'");
+    }
+  }
+
   void ShowChange(const GPSData& data)
   {
     if (data.HasConnected()) {
@@ -890,8 +1404,9 @@ public:
   {
     if (model==GetOpenedAction() && GetOpenedAction()->IsFinished()) {
       InitializeGPS();
-      Lum::OS::display->AddTimer(1,0,refreshTimer);
+      Lum::OS::display->AddTimer(refreshTime,0,refreshTimer);
       Lum::OS::display->AddTimer(reconnectTime,0,reconnectCheckTimer);
+      RefreshTracks();
     }
     else if (model==GetClosedAction() && GetClosedAction()->IsFinished()) {
       DeinitializeGPS();
@@ -903,6 +1418,7 @@ public:
       GPSData dataCopy;
 
       GetGPSDataCopy(dataCopy);
+      AddTrackingRecord(dataCopy);
       ShowChange(dataCopy);
     }
     else if (model==refreshTimer && refreshTimer->IsFinished()) {
@@ -910,15 +1426,46 @@ public:
 
       GetGPSDataCopy(dataCopy);
       ShowChange(dataCopy);
+      AddTrackingRecord(dataCopy);
       UpdateStreams(dataCopy);
 
-      Lum::OS::display->AddTimer(1,0,refreshTimer);
+      Lum::OS::display->AddTimer(refreshTime,0,refreshTimer);
     }
     else if (model==reconnectCheckTimer && reconnectCheckTimer->IsFinished()) {
       ReconnectIfDisconnected();
 
       Lum::OS::display->AddTimer(reconnectTime,0,reconnectCheckTimer);
     }
+    else if (model==startTrackingAction &&
+             startTrackingAction->IsFinished() &&
+             startTrackingAction->IsEnabled()) {
+      StartTracking();
+    }
+    else if (model==stopTrackingAction &&
+             stopTrackingAction->IsFinished() &&
+             stopTrackingAction->IsEnabled()) {
+      StopTracking();
+    }
+    else if (model==deleteTrackAction &&
+             deleteTrackAction->IsFinished() &&
+             deleteTrackAction->IsEnabled()) {
+      DeleteTrack(tracks->GetEntry(trackSelection->GetLine()).GetName());
+    }
+    else if (model==exportTrackAction &&
+             exportTrackAction->IsFinished() &&
+             exportTrackAction->IsEnabled()) {
+      ExportTrack(tracks->GetEntry(trackSelection->GetLine()).GetName());
+    }
+    else if (model==trackSelection) {
+      if (trackSelection->HasSelection()) {
+        deleteTrackAction->Enable();
+        exportTrackAction->Enable();
+      }
+      else {
+        deleteTrackAction->Disable();
+        exportTrackAction->Disable();
+      }
+    }
 
     Dialog::Resync(model,msg);
   }
