@@ -111,6 +111,7 @@ int TimeLine::TransformValue(double y)
 void TimeLine::CalculateVerticalScale()
 {
   if (lastDrawnTop==top && lastDrawnBottom==bottom) {
+    std::cout << "Top/Bottom unchanged => nothing to calculate" << std::endl;
     return;
   }
 
@@ -125,6 +126,11 @@ void TimeLine::CalculateVerticalScale()
   }
 
   verticalScaleStep=pow(10,lrint(log10(std::abs(bottom))-1));
+  std::cout << "Vertical scale step: " << verticalScaleStep << std::endl;
+  if (verticalScaleStep<=1) {
+    std::cout << "(corrected) Vertical scale step: " << verticalScaleStep << std::endl;
+    verticalScaleStep=1;
+  }
 
   while ((TransformValue(bottom)-TransformValue(bottom+verticalScaleStep))<(int)effFontHeight) {
     if (timesFive) {
@@ -136,6 +142,8 @@ void TimeLine::CalculateVerticalScale()
     timesFive=!timesFive;
   }
 
+  std::cout << "Scaled vertical scale step: " << verticalScaleStep << std::endl;
+
   if (log10(verticalScaleStep)<0) {
     verticalScalePrecision=(size_t)ceil(std::abs(log10(verticalScaleStep)));
   }
@@ -143,6 +151,8 @@ void TimeLine::CalculateVerticalScale()
     verticalScalePrecision=0;
   }
 
+  std::cout << "Vertical scale precision: " << verticalScalePrecision << std::endl;
+
   verticalScaleWidth=0;
   for (double i=floor((bottom/verticalScaleStep))*verticalScaleStep; i<=top; i+=verticalScaleStep) {
     Lum::OS::FontExtent extent;
@@ -151,6 +161,8 @@ void TimeLine::CalculateVerticalScale()
 
     verticalScaleWidth=std::max(verticalScaleWidth,extent.width);
   }
+
+  std::cout << "Vertical scale width: " << verticalScaleWidth << std::endl;
 }
 
 void TimeLine::PrintData(Lum::OS::DrawInfo* draw,
@@ -284,7 +296,9 @@ void TimeLine::PrintVerticalScale(Lum::OS::DrawInfo* draw)
   draw->PushForeground(Lum::OS::Display::graphScaleColor);
   draw->PushFont(font);
 
-  for (double i=floor((bottom/verticalScaleStep))*verticalScaleStep; i<=top; i+=verticalScaleStep) {
+  for (double i=floor((bottom/verticalScaleStep))*verticalScaleStep;
+       i<=top;
+       i+=verticalScaleStep) {
     std::wstring tmp=DoubleToWString(i,verticalScalePrecision);
     draw->DrawString(this->x+verticalScaleWidth-font->StringWidth(tmp),
                      TransformValue(i)-font->height/2+this->y+font->ascent,
@@ -320,10 +334,15 @@ void TimeLine::Draw(int x, int y, size_t w, size_t h)
   draw->PushClip(this->x,this->y,width,height);
   draw->PushClip(x,y,w,h);
 
+  std::cout << "Calculate vertical scale..." << std::endl;
   CalculateVerticalScale();
+  std::cout << "Printing data..." << std::endl;
   PrintData(draw,start,end);
+  std::cout << "Printing horizontal scale..." << std::endl;
   PrintHorizontalScale(draw,start);
+  std::cout << "Printing vertical scale..." << std::endl;
   PrintVerticalScale(draw);
+  std::cout << "done." << std::endl;
 
   draw->PopClip();
   draw->PopClip();
@@ -412,9 +431,12 @@ void TimeLine::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
 
     entry.value=this->model->Get(0);
 
+    std::cout << "New value: " << entry.value << std::endl;
+
     data[now]=entry;
 
     if (isnan(entry.value)) {
+      std::cout << "New value is not a number, skipping" << std::endl;
       return;
     }
 
@@ -439,6 +461,8 @@ void TimeLine::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
       }
     }
 
+    std::cout << "Top: " << top << " bottom: " << bottom << std::endl;
+
     Redraw();
   }
 
